<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Controller;

use Dedi\SyliusQuotePlugin\Context\QuoteContextInterface;
use Dedi\SyliusQuotePlugin\Converter\QuoteToCartConverterInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Enum\QuoteActionEnum;
use Dedi\SyliusQuotePlugin\Enum\QuoteEventEnum;
use Dedi\SyliusQuotePlugin\Form\Type\Process\SubmitQuoteType;
use Dedi\SyliusQuotePlugin\Generator\QuotePdfFileGeneratorInterface;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Dedi\SyliusQuotePlugin\Service\QuoteIdSessionManagerInterface;
use Dedi\SyliusQuotePlugin\Voter\QuoteVoter;
use FOS\RestBundle\View\View;
use Sylius\Bundle\ResourceBundle\Controller\ResourceController;
use Sylius\Component\Core\Repository\OrderRepositoryInterface;
use Sylius\Component\Order\Context\CartContextInterface;
use Sylius\Component\Resource\ResourceActions;
use Symfony\Component\EventDispatcher\GenericEvent;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

class QuoteController extends ResourceController
{
    public function widgetAction(Request $request, QuoteContextInterface $quoteContext): Response
    {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        $quote = $quoteContext->getQuote();

        if (!$configuration->isHtmlRequest()) {
            return $this->viewHandler->handle($configuration, View::create($quote));
        }

        return $this->render($configuration->getTemplate('summary.html'), ['quote' => $quote]);
    }

    public function summaryAction(
        Request $request,
        QuoteContextInterface $quoteContext,
        QuoteRepositoryInterface $quoteRepository
    ): Response {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        $quote = $quoteContext->getQuote();
        if (null !== $quote->getId()) {
            $quote = $quoteRepository->findQuoteById($quote->getId());
        }

        if (!$configuration->isHtmlRequest()) {
            return $this->viewHandler->handle($configuration, View::create($quote));
        }

        $form = $this->resourceFormFactory->create($configuration, $quote);

        $submitFormType = $configuration->getParameters()->get('submitForm', SubmitQuoteType::class);

        return $this->render(
            $configuration->getTemplate('summary.html'),
            [
                'quote' => $quote,
                'form' => $form->createView(),
                'submitForm' => $this->createForm($submitFormType)->createView(),
            ]
        );
    }

    public function saveAction(
        Request $request,
        QuoteContextInterface $quoteContext,
        EventDispatcherInterface $eventDispatcher
    ): Response {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        $this->isGrantedOr403($configuration, ResourceActions::UPDATE);
        $resource = $quoteContext->getQuote();

        $form = $this->resourceFormFactory->create($configuration, $resource);

        if (in_array($request->getMethod(), ['POST', 'PUT', 'PATCH'], true) && $form->handleRequest($request)->isValid()) {
            $resource = $form->getData();

            $event = $this->eventDispatcher->dispatchPreEvent(ResourceActions::UPDATE, $configuration, $resource);

            if ($event->isStopped() && !$configuration->isHtmlRequest()) {
                throw new HttpException($event->getErrorCode(), $event->getMessage());
            }
            if ($event->isStopped()) {
                $this->flashHelper->addFlashFromEvent($configuration, $event);

                return $this->redirectHandler->redirectToResource($configuration, $resource);
            }

            if ($configuration->hasStateMachine()) {
                $this->stateMachine->apply($configuration, $resource);
            }

            $this->eventDispatcher->dispatchPostEvent(ResourceActions::UPDATE, $configuration, $resource);

            $eventDispatcher->dispatch(new GenericEvent($resource), QuoteEventEnum::QUOTE_CHANGE);
            $this->manager->flush();

            if (!$configuration->isHtmlRequest()) {
                return $this->viewHandler->handle($configuration, View::create(null, Response::HTTP_NO_CONTENT));
            }

            $this->flashHelper->addSuccessFlash($configuration, ResourceActions::UPDATE, $resource);

            return $this->redirectHandler->redirectToResource($configuration, $resource);
        }

        if (!$configuration->isHtmlRequest()) {
            return $this->viewHandler->handle($configuration, View::create($form, Response::HTTP_BAD_REQUEST));
        }

        return $this->render(
            $configuration->getTemplate(ResourceActions::UPDATE . '.html'),
            [
                'configuration' => $configuration,
                $this->metadata->getName() => $resource,
                'form' => $form->createView(),
                'quote' => $resource,
            ]
        );
    }

    public function clearAction(Request $request, QuoteContextInterface $quoteContext): Response
    {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        $this->isGrantedOr403($configuration, ResourceActions::DELETE);
        $resource = $quoteContext->getQuote();

        if ($configuration->isCsrfProtectionEnabled() && !$this->isCsrfTokenValid((string) $resource->getId(), $request->get('_csrf_token'))) {
            throw new HttpException(Response::HTTP_FORBIDDEN, 'Invalid csrf token.');
        }

        $event = $this->eventDispatcher->dispatchPreEvent(ResourceActions::DELETE, $configuration, $resource);

        if ($event->isStopped() && !$configuration->isHtmlRequest()) {
            throw new HttpException($event->getErrorCode(), $event->getMessage());
        }
        if ($event->isStopped()) {
            $this->flashHelper->addFlashFromEvent($configuration, $event);

            return $this->redirectHandler->redirectToIndex($configuration, $resource);
        }

        $this->repository->remove($resource);
        $this->eventDispatcher->dispatchPostEvent(ResourceActions::DELETE, $configuration, $resource);

        if (!$configuration->isHtmlRequest()) {
            return $this->viewHandler->handle($configuration, View::create(null, Response::HTTP_NO_CONTENT));
        }

        $this->flashHelper->addSuccessFlash($configuration, ResourceActions::DELETE, $resource);

        return $this->redirectHandler->redirectToIndex($configuration, $resource);
    }

    public function confirmationAction(Request $request, QuoteIdSessionManagerInterface $quoteIdSessionManager): Response
    {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        $quoteId = $quoteIdSessionManager->getQuoteIdFromSession();
        if (null === $quoteId) {
            $options = $configuration->getParameters()->get('after_failure');

            return $this->redirectHandler->redirectToRoute(
                $configuration,
                $options['route'] ?? 'sylius_shop_homepage',
                $options['parameters'] ?? []
            );
        }

        $quoteIdSessionManager->removeQuoteIdInSession();
        $quote = $this->repository->find($quoteId);
        if (!$quote instanceof QuoteInterface) {
            throw $this->createNotFoundException();
        }

        return $this->render($configuration->getParameters()->get('template'), ['quote' => $quote]);
    }

    public function downloadPdfAction(
        Request $request,
        string $number,
        QuotePdfFileGeneratorInterface $quotePdfFileGenerator
    ): Response {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        $this->isGrantedOr403($configuration, QuoteActionEnum::DOWNLOAD);

        /** @var QuoteInterface|null $quote */
        $quote = $this->repository->findOneBy(['number' => $number]);
        if (null === $quote) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(QuoteVoter::IS_QUOTE_USABLE, $quote);

        $quotePdf = $quotePdfFileGenerator->generate($quote);

        $response = new Response($quotePdf->getContent(), Response::HTTP_OK, ['Content-Type' => 'application/pdf']);
        $response->headers->add([
            'Content-Disposition' => $response->headers->makeDisposition('inline', $quotePdf->getFileName()),
        ]);

        return $response;
    }

    public function convertToCartAction(
        Request $request,
        CartContextInterface $cartContext,
        OrderRepositoryInterface $orderRepository,
        QuoteToCartConverterInterface $quoteToCartConverter
    ): Response {
        $configuration = $this->requestConfigurationFactory->create($this->metadata, $request);

        /** @var QuoteInterface $quote */
        $quote = $this->findOr404($configuration);

        $this->denyAccessUnlessGranted(QuoteVoter::IS_QUOTE_ACCEPTED, $quote);

        $cart = $cartContext->getCart();
        if (!$cart->isEmpty()) {
            $orderRepository->remove($cart);
            $cart = $cartContext->getCart();
        }

        $quoteToCartConverter->convert($quote, $cart);
        $orderRepository->add($cart);

        $this->flashHelper->addSuccessFlash($configuration, QuoteActionEnum::CONVERT, $quote);

        return $this->redirectHandler->redirectToResource($configuration, $quote);
    }
}
